// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

#include <string>

/**
 * This class represents a texture which is stored on the GPU.
 */

class Texture2D {
public:
    /** Stores the ID of the texture on the GPU */
    unsigned int texture;

    /** Is set to true in the constructor if texture was loaded */
    bool isLoaded = false;

public:
    /**
     * Loads a 2D image from the given file path (1) into the RAM
     * and (2) uploads it to the GPU (Video-RAM) as an image texture.
     */
    Texture2D(std::string imageTexture);

    /**
     * Deletes the texture and ensures that the memory on the GPU
     * is cleaned up.
     */
    ~Texture2D();

    /**
     * Binds the texture so that it can be used in the shader.
     *
     * @textureSlot     Defines the slot where the texture should
     *                  be bound to.
     */
    void bind(int textureSlot = 0);
};
