// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include matrix and vector:
#include <glm/glm.hpp>

// Include a struct for storing uv coordinates:
#include "TexCoord.h"

/**
 * Represents a Vertex with different types of information.
 */
struct Vertex {
    /** Stores the position (X,Y,Z,1) of this vertex */
    glm::vec4 position = glm::vec4(0,0,0,1);

    /** Stores the normal (X,Y,Z,0) of this vertex */
    glm::vec4 normal = glm::vec4(0,0,0,0);

    /** Stores the color (R,G,B,A) of this vertex */
    glm::vec4 color = glm::vec4(0,0,0,1);

    /** Texture coordinate of this vertex */
    TexCoord uv = TexCoord(0,0);

    /** Initializes the attributes with the given parameter values */
    Vertex(glm::vec4 position = glm::vec4(0,0,0,1), glm::vec4 normal = glm::vec4(0,0,0,0), glm::vec4 color = glm::vec4(0,0,0,1), TexCoord uv = TexCoord(0,0))
        : position(position)
        , normal(normal)
        , color(color)
        , uv(uv)
    {}


    std::vector<Vertex*> neighbours;
};
